#!/usr/bin/env python
#
# jabber interface.. to whatever you like
#
# remark:
# jabber connection code is based on the console sample jabber client from
# python-jabber
#
#
# version 0.1 - 20040405
# BeF <bef@erlangen.ccc.de>
#

import socket
from select import select
from string import split,strip,join
import sys,os,re,imp
import ConfigParser

#sys.path.insert(1, os.path.join(sys.path[0], '..'))

import jabber

########## CONFIG ##########

# Change this to 0 if you dont want a color xterm
USE_COLOR = 1

plugindir = os.path.dirname(sys.argv[0]) + '/plugins'


######################################
## don't change anything below unless you know what you're doing

## config file parsing
config = ConfigParser.ConfigParser()
config.read(os.path.dirname(sys.argv[0]) + '/jabberif.conf')

Server = config.get('account', 'server')
Username = config.get('account', 'username')
Password = config.get('account','password')
Resource = config.get('account', 'resource')


## plugins

class pluginhandler:
	def __init__(self):
		self.__plugins = []
		self.__commands = {}
	
	def plugins(self):
		return self.__plugins
	def commands(self):
		return self.__commands

	def load(self, name):
		fp, pathname, description = imp.find_module(name, [plugindir])
		try:
			module = imp.load_module(name, fp, pathname, description)
			self.__plugins.append({'name': name, 'module': module,
				'obj': eval('module.'+name+'(self)')})
		except ImportError:
			print "error loading module", name 
		if fp:
			fp.close()
	
	def register_command(self, module, name, function, help='', args = {}):
		if self.__commands.has_key(name):
			print "error: registration of cmd", name, "failed"
		else:
			print "registering command", '/'+name, "from module", module.name
			self.__commands[name] = {'name': name, 'module': module, 'function': function,
				'help': help, 'args': args}
			
	def send(self, con, to, message):
		msg = jabber.Message(to, message)
		msg.setType('chat')
		con.send(msg)
		
	def send_back(self, con, msg, message):
		self.send(con, msg.getFrom(), message)
	
	def get_config(self, key):
		return config.get('plugins', key)

	def commandhandler(self, args):
		con = args['con']
		msg = args['msg']
		cmd = args['cmd']

		if len(cmd) <= 0:
			return

		if self.__commands.has_key(cmd[0]):
			command = self.__commands[cmd[0]]
			if command['args'].has_key('op_only') and command['args']['op_only'] == 1 and not check_op(msg.getFrom().getStripped()):
				send_back(con, msg, 'sorry. you require op status to execute this command')
			command['function'](args)
		else:
			self.send_back(con, msg, 'error. try /help')


## functions

def check_op(id):
	if userinfo.has_key(id):
		if userinfo[id].has_key('op'):
			if userinfo[id].has_key('op') == 1:
				return 1
	return 0

## message handlers
def messageCB(con, msg):
    """Called when a message is recieved"""
    if msg.getBody():
    	msgbody = strip(msg.getBody())
	if len(msgbody) > 0:
		if  msgbody[0] == '/':
			ph.commandhandler({'con': con, 'msg': msg, 'cmd': split(msgbody[1:], ' ')})
		else:
			ph.commandhandler({'con': con, 'msg': msg, 'cmd': ['msg', msgbody]})

def presenceCB(con, prs):
    """Called when a presence is recieved"""
    who = str(prs.getFrom())
    type = prs.getType()
    if type == None: type = 'available'

    elif type == 'unsubscribe':
        con.send(jabber.Presence(to=who, type='unsubscribed'))
        con.send(jabber.Presence(to=who, type='unsubscribe'))


def iqCB(con,iq):
    """Called when an iq is recieved, we just let the library handle it at the moment"""
    pass

def disconnectedCB(con):
    print colorize("Ouch, network error", 'red')
    sys.exit(1)

## text colorizer
def colorize(txt, col):
    """Return colorized text"""
    if not USE_COLOR: return txt ## DJ - just incase it breaks your terms ;) ##
    cols = { 'red':1, 'green':2, 'yellow':3, 'blue':4}
    initcode = '\033[;3'
    endcode  = '\033[0m'
    if type(col) == type(1): 
        return initcode + str(col) + 'm' + txt + endcode
    try: return initcode + str(cols[col]) + 'm' + txt + endcode
    except: return txt



##################################################

## load user info
userinfo = eval(config.get('users', 'info'))

## load plugins
ph = pluginhandler()

for plugin in eval(config.get('plugins','startup')):
	ph.load(plugin)

##################################################


## non-ssl connect
#con = jabber.Client(host=Server,debug=1 ,log=sys.stderr)
## ssl connect
con = jabber.Client(host=Server, port=5223, connection=jabber.xmlstream.TCP_SSL)

try:
    con.connect()
except IOError, e:
    print "Couldn't connect: %s" % e
    sys.exit(0)
else:
    print colorize("Connected",'red')

## handlers
con.registerHandler('message', messageCB)
con.registerHandler('presence', presenceCB)
con.registerHandler('iq', iqCB)
con.setDisconnectHandler(disconnectedCB)


## login
print colorize("Attempting to log in...", 'red')

if con.auth(Username,Password,Resource):
    print colorize("Logged in as %s to server %s" % ( Username, Server), 'red')
else:
    print "eek -> ", con.lastErr, con.lastErrCode
    sys.exit(1)

## roster/presence init
print colorize("Requesting Roster Info" , 'red')
con.requestRoster()
con.sendInitPresence()

JID = Username + '@' + Server + '/' + Resource

## loop forever
while(1):
	try:
		con.process(1)
	except socket.sslerror:
		sys.exit(0)
#	except:
#		print "unknown error"
#		sys.exit(1)
    
        
